@@ -35,16 +35,19 @@ import pandas as pd
 import torch
 from tabulate import tabulate
 
-from ludwig.backend import Backend, initialize_backend
+from ludwig.backend import Backend, initialize_backend, provision_preprocessing_workers
 from ludwig.callbacks import Callback
 from ludwig.constants import (
     AUTO,
     BATCH_SIZE,
+    DEFAULTS,
     EVAL_BATCH_SIZE,
     FULL,
     HYPEROPT,
     HYPEROPT_WARNING,
     LEARNING_RATE,
+    MIN_VALIDATION_SET_ROWS,
+    MODEL_TYPE,
     PREPROCESSING,
     TEST,
     TRAINER,
@@ -56,27 +59,26 @@ from ludwig.data.postprocessing import convert_predictions, postprocess
 from ludwig.data.preprocessing import load_metadata, preprocess_for_prediction, preprocess_for_training
 from ludwig.features.feature_registries import update_config_with_metadata
 from ludwig.globals import (
-    INFERENCE_MODULE_FILE_NAME,
     LUDWIG_VERSION,
     MODEL_HYPERPARAMETERS_FILE_NAME,
-    MODEL_WEIGHTS_FILE_NAME,
     set_disable_progressbar,
     TRAIN_SET_METADATA_FILE_NAME,
 )
+from ludwig.models.base import BaseModel
 from ludwig.models.calibrator import Calibrator
-from ludwig.models.ecd import ECD
-from ludwig.models.inference import InferenceModule
+from ludwig.models.inference import InferenceModule, save_ludwig_model_for_inference
 from ludwig.models.predictor import (
     calculate_overall_stats,
     print_evaluation_stats,
     save_evaluation_stats,
     save_prediction_outputs,
 )
-from ludwig.models.trainer import Trainer
-from ludwig.modules.metric_modules import get_best_function
+from ludwig.models.registry import model_type_registry
 from ludwig.schema import validate_config
-from ludwig.schema.utils import load_config_with_kwargs
+from ludwig.schema.utils import load_trainer_with_kwargs
 from ludwig.utils import metric_utils
+from ludwig.utils.backward_compatibility import upgrade_to_latest_version
+from ludwig.utils.config_utils import merge_config_preprocessing_with_feature_specific_defaults
 from ludwig.utils.data_utils import (
     figure_data_format,
     generate_kfold_splits,
@@ -85,11 +87,19 @@ from ludwig.utils.data_utils import (
     load_yaml,
     save_json,
 )
+from ludwig.utils.dataset_utils import generate_dataset_statistics
 from ludwig.utils.defaults import default_random_seed, merge_with_defaults
-from ludwig.utils.fs_utils import makedirs, open_file, path_exists, upload_output_directory
-from ludwig.utils.misc_utils import get_file_names, get_output_directory, set_saved_weights_in_checkpoint_flag
+from ludwig.utils.fs_utils import makedirs, path_exists, upload_output_directory
+from ludwig.utils.misc_utils import (
+    get_file_names,
+    get_from_registry,
+    get_output_directory,
+    set_saved_weights_in_checkpoint_flag,
+)
 from ludwig.utils.print_utils import print_boxed
-from ludwig.utils.torch_utils import get_torch_device
+from ludwig.utils.torch_utils import DEVICE
+from ludwig.utils.trainer_utils import get_training_report
+from ludwig.utils.types import TorchDevice
 
 logger = logging.getLogger(__name__)
 
@@ -205,9 +215,13 @@ class LudwigModel:
             config_dict = copy.deepcopy(config)
             self.config_fp = None
 
-        # merge config with defaults
-        self.base_config = copy.deepcopy(config_dict)
-        self.config = merge_with_defaults(config_dict)
+        self.base_config = config_dict
+
+        # Upgrades deprecated fields and adds new required fields in case the config loaded from disk is old.
+        upgraded_config = upgrade_to_latest_version(config_dict)
+
+        # Merge upgraded config with defaults.
+        self.config = merge_with_defaults(upgraded_config)
         validate_config(self.config)
 
         # setup logging
@@ -460,19 +474,15 @@ class LudwigModel:
             self.training_set_metadata = training_set_metadata
 
             if self.backend.is_coordinator():
-                dataset_statistics = [["Dataset", "Size"]]
-                dataset_statistics.append(["Training", len(training_set)])
-                if validation_set is not None:
-                    dataset_statistics.append(["Validation", len(validation_set)])
-                if test_set is not None:
-                    dataset_statistics.append(["Test", len(test_set)])
+                dataset_statistics = generate_dataset_statistics(training_set, validation_set, test_set)
+
                 if not skip_save_model:
                     # save train set metadata
                     os.makedirs(model_dir, exist_ok=True)
                     save_json(os.path.join(model_dir, TRAIN_SET_METADATA_FILE_NAME), training_set_metadata)
 
-                logger.info("\nDataset sizes:")
-                logger.info(tabulate(dataset_statistics, headers="firstrow", tablefmt="fancy_grid", floatfmt=".4f"))
+                logger.info("\nDataset Statistics")
+                logger.info(tabulate(dataset_statistics, headers="firstrow", tablefmt="fancy_grid"))
 
             for callback in self.callbacks:
                 callback.on_train_init(
@@ -495,8 +505,8 @@ class LudwigModel:
                 self.model = LudwigModel.create_model(self.config, random_seed=random_seed)
                 set_saved_weights_in_checkpoint_flag(self.config)
 
-            # Convert config dictionary into an instance of TrainerConfig.
-            trainer_config, _ = load_config_with_kwargs(Trainer.get_schema_cls(), self.config[TRAINER])
+            # Convert config dictionary into an instance of BaseTrainerConfig.
+            trainer_config, _ = load_trainer_with_kwargs(self.config[MODEL_TYPE], self.config[TRAINER])
 
             with self.backend.create_trainer(
                 model=self.model,
@@ -509,7 +519,7 @@ class LudwigModel:
                 random_seed=random_seed,
             ) as trainer:
                 # auto tune batch size
-                if self.config[TRAINER][BATCH_SIZE] == AUTO or self.config[TRAINER][EVAL_BATCH_SIZE] == AUTO:
+                if self.config[TRAINER].get(BATCH_SIZE, None) == AUTO or self.config[TRAINER][EVAL_BATCH_SIZE] == AUTO:
                     # TODO (ASN): add support for substitute_with_max parameter
                     # TODO(travis): detect train and eval batch sizes separately (enable / disable gradients)
                     tuned_batch_size = trainer.tune_batch_size(self.config, training_set, random_seed=random_seed)
@@ -559,18 +569,24 @@ class LudwigModel:
                             self.backend,
                             batch_size=trainer.eval_batch_size,
                         )
-                        if validation_set is not None:
-                            # Use backend.createPredictor to ensure we get ray predictor with ray backend
-                            calibrator.train_calibration(validation_set, VALIDATION)
-                        else:
+                        if validation_set is None:
                             logger.warning(
-                                "Calibration uses validation set, but not validation split specified. "
+                                "Calibration uses validation set, but no validation split specified."
                                 "Will use training set for calibration."
+                                "Recommend providing a validation set when using calibration."
                             )
                             calibrator.train_calibration(training_set, TRAINING)
+                        elif len(validation_set) < MIN_VALIDATION_SET_ROWS:
+                            logger.warning(
+                                f"Validation set size ({len(validation_set)} rows) is too small for calibration."
+                                "Will use training set for calibration."
+                                f"Validation set much have at least {MIN_VALIDATION_SET_ROWS} rows."
+                            )
+                            calibrator.train_calibration(training_set, TRAINING)
+                        else:
+                            calibrator.train_calibration(validation_set, VALIDATION)
                         if not skip_save_model:
-                            model_weights_path = os.path.join(model_dir, MODEL_WEIGHTS_FILE_NAME)
-                            torch.save(self.model.state_dict(), model_weights_path)
+                            self.model.save(model_dir)
 
                     # Unpack train()'s return.
                     # The statistics are all nested dictionaries of TrainerMetrics: feature_name -> metric_name ->
@@ -589,44 +605,17 @@ class LudwigModel:
                         if not skip_save_training_statistics and path_exists(os.path.dirname(training_stats_fn)):
                             save_json(training_stats_fn, train_stats)
 
-                    # grab the results of the model with highest validation test performance
-                    validation_field = trainer.validation_field
-                    validation_metric = trainer.validation_metric
-                    validation_field_result = train_valiset_stats[validation_field]
-
-                    best_function = get_best_function(validation_metric)
-
                     # results of the model with highest validation test performance
                     if self.backend.is_coordinator() and validation_set is not None:
                         print_boxed("TRAINING REPORT")
-                        best_vali_index, (
-                            epoch_best_validation_metric,
-                            step_best_validation_metric,
-                            best_validation_metric,
-                        ) = best_function(
-                            enumerate(validation_field_result[validation_metric]),
-                            # -1 for the last element of the TrainerMetric namedtuple.
-                            key=lambda index_epoch_step_value: index_epoch_step_value[1][-1],
+                        training_report = get_training_report(
+                            trainer.validation_field,
+                            trainer.validation_metric,
+                            test_set is not None,
+                            train_valiset_stats,
+                            train_testset_stats,
                         )
-                        logger.info(
-                            f"Best validation model step: {step_best_validation_metric}, epoch: "
-                            f"{epoch_best_validation_metric + 1}"
-                        )
-                        logger.info(
-                            f"Best validation model {validation_metric} on validation set {validation_field}: "
-                            f"{best_validation_metric}"
-                        )
-                        if test_set is not None:
-                            validation_selected_test_metric_score = train_testset_stats[validation_field][
-                                validation_metric
-                            ][best_vali_index][
-                                -1
-                            ]  # -1 for the last element of the TrainerMetric namedtuple.
-
-                            logger.info(
-                                f"Best validation model {validation_metric} on test set {validation_field}: "
-                                f"{validation_selected_test_metric_score}"
-                            )
+                        logger.info(tabulate(training_report, tablefmt="fancy_grid"))
                         logger.info(f"\nFinished: {experiment_name}_{model_name}")
                         logger.info(f"Saved to: {output_directory}")
                 finally:
@@ -637,10 +626,7 @@ class LudwigModel:
 
                 # Ensure model weights are saved to the driver if training was done remotely
                 if self.backend.is_coordinator() and not skip_save_model:
-                    weights_save_path = os.path.join(model_dir, MODEL_WEIGHTS_FILE_NAME)
-                    if not path_exists(weights_save_path):
-                        with open_file(weights_save_path, "wb") as f:
-                            torch.save(self.model.state_dict(), f)
+                    self.model.save(model_dir)
 
                 # Synchronize model weights between workers
                 self.backend.sync_model(self.model)
@@ -685,18 +671,24 @@ class LudwigModel:
         :return: (None) `None`
         """
         training_set_metadata = training_set_metadata or self.training_set_metadata
-        training_dataset, _, _, training_set_metadata = preprocess_for_training(
-            self.config,
-            training_set=dataset,
-            training_set_metadata=training_set_metadata,
-            data_format=data_format,
-            skip_save_processed_input=True,
-            preprocessing_params=self.config[PREPROCESSING],
-            backend=self.backend,
-            random_seed=random_seed,
-            callbacks=self.callbacks,
+
+        preprocessing_params = merge_config_preprocessing_with_feature_specific_defaults(
+            self.config.get(PREPROCESSING, {}), self.config.get(DEFAULTS, {})
         )
 
+        with provision_preprocessing_workers(self.backend):
+            training_dataset, _, _, training_set_metadata = preprocess_for_training(
+                self.config,
+                training_set=dataset,
+                training_set_metadata=training_set_metadata,
+                data_format=data_format,
+                skip_save_processed_input=True,
+                preprocessing_params=preprocessing_params,
+                backend=self.backend,
+                random_seed=random_seed,
+                callbacks=self.callbacks,
+            )
+
         if not self.training_set_metadata:
             self.training_set_metadata = training_set_metadata
 
@@ -706,7 +698,7 @@ class LudwigModel:
             set_saved_weights_in_checkpoint_flag(self.config)
 
         if not self._online_trainer:
-            config, _ = load_config_with_kwargs(Trainer.get_schema_cls(), self.config[TRAINER])
+            config, _ = load_trainer_with_kwargs(self.config[MODEL_TYPE], self.config[TRAINER])
             self._online_trainer = self.backend.create_trainer(config=config, model=self.model, random_seed=random_seed)
 
         self.model = self._online_trainer.train_online(training_dataset)
@@ -802,9 +794,8 @@ class LudwigModel:
                 skip_save_unprocessed_output=skip_save_unprocessed_output or not self.backend.is_coordinator(),
             )
             converted_postproc_predictions = convert_predictions(
-                postproc_predictions, self.model.output_features, return_type=return_type
+                postproc_predictions, self.model.output_features, return_type=return_type, backend=self.backend
             )
-
             if self.backend.is_coordinator():
                 if not skip_save_predictions:
                     save_prediction_outputs(
@@ -958,9 +949,7 @@ class LudwigModel:
 
             if collect_predictions:
                 postproc_predictions = convert_predictions(
-                    postproc_predictions,
-                    self.model.output_features,
-                    return_type=return_type,
+                    postproc_predictions, self.model.output_features, return_type=return_type, backend=self.backend
                 )
 
             for callback in self.callbacks:
@@ -1296,21 +1285,27 @@ class LudwigModel:
             `(proc_training_set, proc_validation_set, proc_test_set, training_set_metadata)`.
         """
         print_boxed("PREPROCESSING")
-        preprocessed_data = preprocess_for_training(
-            self.config,
-            dataset=dataset,
-            training_set=training_set,
-            validation_set=validation_set,
-            test_set=test_set,
-            training_set_metadata=training_set_metadata,
-            data_format=data_format,
-            skip_save_processed_input=skip_save_processed_input,
-            preprocessing_params=self.config[PREPROCESSING],
-            backend=self.backend,
-            random_seed=random_seed,
-            callbacks=self.callbacks,
+
+        preprocessing_params = merge_config_preprocessing_with_feature_specific_defaults(
+            self.config.get(PREPROCESSING, {}), self.config.get(DEFAULTS, {})
         )
 
+        with provision_preprocessing_workers(self.backend):
+            preprocessed_data = preprocess_for_training(
+                self.config,
+                dataset=dataset,
+                training_set=training_set,
+                validation_set=validation_set,
+                test_set=test_set,
+                training_set_metadata=training_set_metadata,
+                data_format=data_format,
+                skip_save_processed_input=skip_save_processed_input,
+                preprocessing_params=preprocessing_params,
+                backend=self.backend,
+                random_seed=random_seed,
+                callbacks=self.callbacks,
+            )
+
         (proc_training_set, proc_validation_set, proc_test_set, training_set_metadata) = preprocessed_data
 
         return proc_training_set, proc_validation_set, proc_test_set, training_set_metadata
@@ -1369,7 +1364,9 @@ class LudwigModel:
 
         config = backend.broadcast_return(lambda: load_json(os.path.join(model_dir, MODEL_HYPERPARAMETERS_FILE_NAME)))
 
-        # Upgrades deprecated fields and adds new required fields, in case the config loaded from disk is old.
+        # Upgrades deprecated fields and adds new required fields in case the config loaded from disk is old.
+        config = upgrade_to_latest_version(config)
+        # Merge upgraded config with defaults.
         config = merge_with_defaults(config)
 
         if backend_param is None and "backend" in config:
@@ -1421,9 +1418,7 @@ class LudwigModel:
         ```
         """
         if self.backend.is_coordinator():
-            weights_save_path = os.path.join(model_dir, MODEL_WEIGHTS_FILE_NAME)
-            device = torch.device(get_torch_device())
-            self.model.load_state_dict(torch.load(weights_save_path, map_location=device))
+            self.model.load(model_dir)
 
         self.backend.sync_model(self.model)
 
@@ -1453,8 +1448,7 @@ class LudwigModel:
         self.save_config(save_path)
 
         # save model weights
-        model_weights_path = os.path.join(save_path, MODEL_WEIGHTS_FILE_NAME)
-        torch.save(self.model.state_dict(), model_weights_path)
+        self.model.save(save_path)
 
         # save training set metadata
         training_set_metadata_path = os.path.join(save_path, TRAIN_SET_METADATA_FILE_NAME)
@@ -1475,43 +1469,65 @@ class LudwigModel:
         model_hyperparameters_path = os.path.join(save_path, MODEL_HYPERPARAMETERS_FILE_NAME)
         save_json(model_hyperparameters_path, self.config)
 
-    def to_torchscript(self, model_only: bool = False):
-        """Converts the trained LudwigModule, including preprocessing and postprocessing, to Torchscript.
-
-        The scripted module takes in a `Dict[str, Union[List[str], Tensor]]` as input.
-
-        More specifically, for every input feature, we provide either a Tensor of batch_size inputs, a list of Tensors
-        batch_size in length, or a list of strings batch_size in length.
-
-        Note that the dimensions of all Tensors and lengths of all lists must match.
-
-        Similarly, the output will be a dictionary of dictionaries, where each feature has its own dictionary of
-        outputs. The outputs will be a list of strings for predictions with string types, while other outputs will be
-        tensors of varying dimensions for probabilities, logits, etc.
+    def to_torchscript(
+        self,
+        model_only: bool = False,
+        device: Optional[TorchDevice] = None,
+    ):
+        """Converts the trained model to Torchscript.
 
         Args:
             model_only (bool, optional): If True, only the ECD model will be converted to Torchscript. Else,
-                preprocessing and postprocessing will also be converted to Torchscript.
+                preprocessing and postprocessing steps will also be converted to Torchscript.
+            device (TorchDevice, optional): If None, the model will be converted to Torchscript on the same device to
+                ensure maximum model parity.
+        Returns:
+            A torch.jit.ScriptModule that can be used to predict on a dictionary of inputs.
         """
+        if device is None:
+            device = DEVICE
+
         self._check_initialization()
         if model_only:
-            return self.model.to_torchscript()
+            return self.model.to_torchscript(device)
         else:
-            inference_module = InferenceModule(self.model, self.config, self.training_set_metadata)
+            inference_module = InferenceModule.from_ludwig_model(
+                self.model, self.config, self.training_set_metadata, device=device
+            )
             return torch.jit.script(inference_module)
 
-    def save_torchscript(self, save_path: str, model_only: bool = False):
-        """Saves the Torchscript model to disk."""
-        inference_module = self.to_torchscript(model_only=model_only)
-        inference_module.save(os.path.join(save_path, INFERENCE_MODULE_FILE_NAME))
+    def save_torchscript(
+        self,
+        save_path: str,
+        model_only: bool = False,
+        device: Optional[TorchDevice] = None,
+    ):
+        """Saves the Torchscript model to disk.
+
+        save_path (str): The path to the directory where the model will be saved. model_only (bool, optional): If True,
+        only the ECD model will be converted to Torchscript. Else, the     preprocessing and postprocessing steps will
+        also be converted to Torchscript. device (TorchDevice, optional): If None, the model will be converted to
+        Torchscript on the same device to     ensure maximum model parity.
+        """
+        if device is None:
+            device = DEVICE
+
+        save_ludwig_model_for_inference(
+            save_path,
+            self.model,
+            self.config,
+            self.training_set_metadata,
+            model_only=model_only,
+            device=device,
+        )
 
     def _check_initialization(self):
         if self.model is None or self.config is None or self.training_set_metadata is None:
             raise ValueError("Model has not been trained or loaded")
 
     @staticmethod
-    def create_model(config: dict, random_seed: int = default_random_seed) -> ECD:
-        """Instantiates Encoder-Combiner-Decoder (ECD) object.
+    def create_model(config: dict, random_seed: int = default_random_seed) -> BaseModel:
+        """Instantiates BaseModel object.
 
         # Inputs
         :param config: (dict) Ludwig config
@@ -1520,15 +1536,10 @@ class LudwigModel:
             splits and any other random function.
 
         # Return
-        :return: (ludwig.models.ECD) Instance of the Ludwig model object.
+        :return: (ludwig.models.BaseModel) Instance of the Ludwig model object.
         """
-        # todo: support loading other model types based on config
-        return ECD(
-            input_features_def=config["input_features"],
-            combiner_def=config["combiner"],
-            output_features_def=config["output_features"],
-            random_seed=random_seed,
-        )
+        model_type = get_from_registry(config[MODEL_TYPE], model_type_registry)
+        return model_type(**config, random_seed=random_seed)
 
     @staticmethod
     def set_logging_level(logging_level: int) -> None:
