@@ -18,7 +18,7 @@ import contextlib
 import copy
 import logging
 from functools import partial
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from typing import Any, Callable, Dict, List, Optional, Tuple, TYPE_CHECKING, Union
 
 import dask
 import numpy as np
@@ -26,21 +26,34 @@ import pandas as pd
 import ray
 import torch
 import tqdm
+from fsspec.config import conf
 from packaging import version
+from pyarrow.fs import FSSpecHandler, PyFileSystem
 from ray import ObjectRef
 from ray.data.dataset_pipeline import DatasetPipeline
-from ray.data.extensions import TensorDtype
 from ray.util.dask import ray_dask_get
+from ray.util.placement_group import placement_group, remove_placement_group
+
+if TYPE_CHECKING:
+    from ludwig.api import LudwigModel
 
 from ludwig.backend.base import Backend, RemoteTrainingMixin
-from ludwig.constants import NAME, PREPROCESSING, PROC_COLUMN
+from ludwig.backend.datasource import BinaryIgnoreNoneTypeDatasource
+from ludwig.constants import MODEL_ECD, MODEL_GBM, NAME, PREPROCESSING, PROC_COLUMN, TYPE
 from ludwig.data.dataframe.base import DataFrameEngine
-from ludwig.data.dataset.ray import RayDataset, RayDatasetManager, RayDatasetShard
+from ludwig.data.dataset.ray import _SCALAR_TYPES, cast_as_tensor_dtype, RayDataset, RayDatasetManager, RayDatasetShard
+from ludwig.models.base import BaseModel
 from ludwig.models.ecd import ECD
 from ludwig.models.predictor import BasePredictor, get_output_columns, Predictor, RemotePredictor
-from ludwig.models.trainer import BaseTrainer, RemoteTrainer, TrainerConfig
-from ludwig.utils.fs_utils import get_bytes_obj_if_path
+from ludwig.schema.trainer import ECDTrainerConfig, GBMTrainerConfig
+from ludwig.trainers.registry import ray_trainers_registry, register_ray_trainer
+from ludwig.trainers.trainer import BaseTrainer, RemoteTrainer
+from ludwig.utils.data_utils import use_credentials
+from ludwig.utils.dataframe_utils import set_index_name
+from ludwig.utils.fs_utils import get_fs_and_path
 from ludwig.utils.horovod_utils import initialize_horovod
+from ludwig.utils.misc_utils import get_from_registry
+from ludwig.utils.system_utils import Resources
 from ludwig.utils.torch_utils import get_torch_device, initialize_pytorch
 from ludwig.utils.types import Series
 
@@ -61,6 +74,8 @@ if _ray112:
     from ludwig.backend._ray112_compat import HorovodConfig
 else:
     from ray.train.horovod import HorovodConfig
+RAY_DEFAULT_PARALLELISM = 200
+FIFTEEN_MINS_IN_S = 15 * 60
 
 
 # TODO: deprecated v0.5
@@ -316,8 +331,16 @@ class TqdmCallback(rt.TrainingCallback):
                 self.progess_bars[_id].update(update_by)
 
 
+@register_ray_trainer("trainer", MODEL_ECD, default=True)
 class RayTrainerV2(BaseTrainer):
-    def __init__(self, model, trainer_kwargs, data_loader_kwargs, executable_kwargs):
+    def __init__(
+        self,
+        model: BaseModel,
+        trainer_kwargs: Dict[str, Any],
+        data_loader_kwargs: Dict[str, Any],
+        executable_kwargs: Dict[str, Any],
+        **kwargs,
+    ):
         self.model = model.cpu()
         self.data_loader_kwargs = data_loader_kwargs
         self.executable_kwargs = executable_kwargs
@@ -325,6 +348,10 @@ class RayTrainerV2(BaseTrainer):
         self._validation_field = None
         self._validation_metric = None
 
+    @staticmethod
+    def get_schema_cls():
+        return ECDTrainerConfig
+
     @contextlib.contextmanager
     def create_runner(self):
         trainer = Trainer(**{**get_trainer_kwargs(), **self.trainer_kwargs})
@@ -417,7 +444,7 @@ class RayTrainerV2(BaseTrainer):
         return self._validation_metric
 
     @property
-    def config(self) -> TrainerConfig:
+    def config(self) -> ECDTrainerConfig:
         return self.executable_kwargs["config"]
 
     @property
@@ -502,8 +529,9 @@ class HorovodRemoteTrainer(RemoteTrainer):
         super().__init__(horovod=horovod, **kwargs)
 
 
+@register_ray_trainer("ray_legacy_trainer", MODEL_ECD)
 class RayLegacyTrainer(BaseTrainer):
-    def __init__(self, horovod_kwargs, executable_kwargs):
+    def __init__(self, horovod_kwargs: Dict[str, Any], executable_kwargs: Dict[str, Any], **kwargs):
         # TODO ray: make this more configurable by allowing YAML overrides of timeout_s, etc.
         if RayExecutor is None:
             logger.error(
@@ -515,6 +543,10 @@ class RayLegacyTrainer(BaseTrainer):
         self.executor = RayExecutor(setting, **{**get_horovod_kwargs(), **horovod_kwargs})
         self.executor.start(executable_cls=HorovodRemoteTrainer, executable_kwargs=executable_kwargs)
 
+    @staticmethod
+    def get_schema_cls():
+        return ECDTrainerConfig
+
     def train(self, model, training_set, validation_set=None, test_set=None, **kwargs):
         workers = self.executor.driver.workers
         train_shards = training_set.pipeline().split(n=len(workers), locality_hints=workers, equal=True)
@@ -589,7 +621,9 @@ def eval_fn(
 
 
 class RayPredictor(BasePredictor):
-    def __init__(self, model: ECD, df_engine: DataFrameEngine, trainer_kwargs, data_loader_kwargs, **predictor_kwargs):
+    def __init__(
+        self, model: BaseModel, df_engine: DataFrameEngine, trainer_kwargs, data_loader_kwargs, **predictor_kwargs
+    ):
         self.batch_size = predictor_kwargs["batch_size"]
         self.trainer_kwargs = trainer_kwargs
         self.data_loader_kwargs = data_loader_kwargs
@@ -599,10 +633,10 @@ class RayPredictor(BasePredictor):
         self.df_engine = df_engine
 
     def get_trainer_kwargs(self) -> Dict[str, Any]:
-        return {**self.trainer_kwargs, **get_trainer_kwargs()}
+        return {**get_trainer_kwargs(), **self.trainer_kwargs}
 
     def get_resources_per_worker(self) -> Tuple[int, int]:
-        trainer_kwargs = {**self.trainer_kwargs, **get_trainer_kwargs()}
+        trainer_kwargs = self.get_trainer_kwargs()
         resources_per_worker = trainer_kwargs.get("resources_per_worker", {})
         num_gpus = resources_per_worker.get("GPU", 0)
         num_cpus = resources_per_worker.get("CPU", (1 if num_gpus == 0 else 0))
@@ -628,11 +662,9 @@ class RayPredictor(BasePredictor):
 
         def to_tensors(df: pd.DataFrame) -> pd.DataFrame:
             for c in columns:
-                df[c] = df[c].astype(TensorDtype())
+                df[c] = cast_as_tensor_dtype(df[c])
             return df
 
-        # TODO(shreya): self.trainer_kwargs should have the correct resources; debug.
-        # trainer_kwargs = {**get_trainer_kwargs(), **self.trainer_kwargs}
         num_cpus, num_gpus = self.get_resources_per_worker()
 
         predictions = dataset.ds.map_batches(to_tensors, batch_format="pandas").map_batches(
@@ -736,15 +768,20 @@ class RayPredictor(BasePredictor):
 
             def __call__(self, df: pd.DataFrame) -> pd.DataFrame:
                 dataset = self._prepare_batch(df)
-                predictions = self.predict(batch=dataset)
-
+                predictions = self.predict(batch=dataset).set_index(df.index)
                 for output_feature in self.model.output_features.values():
                     predictions = output_feature.flatten(predictions)
                 ordered_predictions = predictions[self.output_columns]
                 return ordered_predictions
 
             def _prepare_batch(self, batch: pd.DataFrame) -> Dict[str, np.ndarray]:
-                res = {c: batch[c].to_numpy() for c in self.features.keys()}
+                res = {}
+                for c in self.features.keys():
+                    if self.features[c][TYPE] not in _SCALAR_TYPES:
+                        # Ensure columns stacked instead of turned into np.array([np.array, ...], dtype=object) objects
+                        res[c] = np.stack(batch[c].values)
+                    else:
+                        res[c] = batch[c].to_numpy()
 
                 for c in self.features.keys():
                     reshape = self.reshape_map.get(c)
@@ -757,46 +794,94 @@ class RayPredictor(BasePredictor):
 
 
 class RayBackend(RemoteTrainingMixin, Backend):
-    def __init__(self, processor=None, trainer=None, loader=None, use_legacy=False, **kwargs):
+    BACKEND_TYPE = "ray"
+
+    def __init__(self, processor=None, trainer=None, loader=None, use_legacy=False, preprocessor_kwargs=None, **kwargs):
         super().__init__(dataset_manager=RayDatasetManager(self), **kwargs)
+        self._preprocessor_kwargs = preprocessor_kwargs or {}
         self._df_engine = _get_df_engine(processor)
         self._horovod_kwargs = trainer or {}
         self._pytorch_kwargs = {}
         self._data_loader_kwargs = loader or {}
         self._use_legacy = use_legacy
+        self._preprocessor_pg = None
 
     def initialize(self):
-        if not ray.is_initialized():
-            try:
-                ray.init("auto", ignore_reinit_error=True)
-            except ConnectionError:
-                logger.info("Initializing new Ray cluster...")
-                ray.init(ignore_reinit_error=True)
+        initialize_ray()
 
         dask.config.set(scheduler=ray_dask_get)
         # Disable placement groups on dask
         dask.config.set(annotations={"ray_remote_args": {"placement_group": None}})
 
+    def generate_bundles(self, num_cpu):
+        # Ray requires that each bundle be scheduleable on a single node.
+        # So a bundle of 320 cpus would never get scheduled. For now a simple heuristic
+        # to be used is to just request 1 cpu at a time.
+        return [{"CPU": 1} for _ in range(int(num_cpu))]
+
+    @contextlib.contextmanager
+    def provision_preprocessing_workers(self):
+        num_cpu = self._preprocessor_kwargs.get("num_cpu")
+        if not num_cpu:
+            logger.info(
+                "Backend config has num_cpu not set." " provision_preprocessing_workers() is a no-op in this case."
+            )
+            yield
+        else:
+            bundles = self.generate_bundles(num_cpu)
+            logger.info("Requesting bundles of %s for preprocessing", bundles)
+            self._preprocessor_pg = placement_group(bundles)
+            ready = self._preprocessor_pg.wait(FIFTEEN_MINS_IN_S)
+
+            if not ready:
+                remove_placement_group(self._preprocessor_pg)
+                raise TimeoutError(
+                    "Ray timed out in provisioning the placement group for preprocessing."
+                    f" {num_cpu} CPUs were requested but were unable to be provisioned."
+                )
+
+            logger.info("%s CPUs were requested and successfully provisioned", num_cpu)
+            try:
+                with dask.config.set(annotations={"ray_remote_args": {"placement_group": self._preprocessor_pg}}):
+                    yield
+            finally:
+                self._release_preprocessing_workers()
+
+    def _release_preprocessing_workers(self):
+        if self._preprocessor_pg is not None:
+            remove_placement_group(self._preprocessor_pg)
+        self._preprocessor_pg = None
+
     def initialize_pytorch(self, **kwargs):
         # Make sure we don't claim any GPU resources on the head node
         initialize_pytorch(gpus=-1)
         self._pytorch_kwargs = kwargs
 
-    def create_trainer(self, model: ECD, **kwargs):
+    def create_trainer(self, model: BaseModel, **kwargs) -> "BaseTrainer":  # noqa: F821
         executable_kwargs = {**kwargs, **self._pytorch_kwargs}
         if not self._use_legacy:
+            trainers_for_model = get_from_registry(model.type(), ray_trainers_registry)
+
+            config: Union[ECDTrainerConfig, GBMTrainerConfig] = kwargs["config"]
+            trainer_cls = get_from_registry(config.type, trainers_for_model)
+
             # Deep copy to workaround https://github.com/ray-project/ray/issues/24139
-            return RayTrainerV2(
-                model,
-                copy.deepcopy(self._horovod_kwargs),
-                self._data_loader_kwargs,
-                executable_kwargs,
-            )
+            all_kwargs = {
+                "model": model,
+                "trainer_kwargs": copy.deepcopy(self._horovod_kwargs),
+                "data_loader_kwargs": self._data_loader_kwargs,
+                "executable_kwargs": executable_kwargs,
+            }
+            all_kwargs.update(kwargs)
+            return trainer_cls(**all_kwargs)
         else:
+            if model.name == MODEL_GBM:
+                raise RuntimeError("Legacy trainer not supported for GBM models.")
+
             # TODO: deprecated 0.5
             return RayLegacyTrainer(self._horovod_kwargs, executable_kwargs)
 
-    def create_predictor(self, model: ECD, **kwargs):
+    def create_predictor(self, model: BaseModel, **kwargs):
         executable_kwargs = {**kwargs, **self._pytorch_kwargs}
         return RayPredictor(
             model,
@@ -824,21 +909,94 @@ class RayBackend(RemoteTrainingMixin, Backend):
                 f"Set preprocessing config `in_memory: True` for feature {feature[NAME]}"
             )
 
-    def read_binary_files(self, column: Series, map_fn: Optional[Callable] = None) -> Series:
-        ds = self.df_engine.to_ray_dataset(column.to_frame(name=column.name))
+    def read_binary_files(
+        self, column: Series, map_fn: Optional[Callable] = None, file_size: Optional[int] = None
+    ) -> Series:
+        column = column.fillna(np.nan).replace([np.nan], [None])  # normalize NaNs to None
+
+        # Assume that the list of filenames is small enough to fit in memory. Should be true unless there
+        # are literally billions of filenames.
+        # TODO(travis): determine if there is a performance penalty to passing in individual files instead of
+        #  a directory. If so, we can do some preprocessing to determine if it makes sense to read the full directory
+        #  then filter out files as a postprocessing step (depending on the ratio of included to excluded files in
+        #  the directory). Based on a preliminary look at how Ray handles directory expansion to files, it looks like
+        #  there should not be any difference between providing a directory versus a list of files.
+        pd_column = self.df_engine.compute(column)
+        fnames = pd_column.values.tolist()
+        idxs = pd_column.index.tolist()
+
+        # Sample a filename to extract the filesystem info
+        sample_fname = fnames[0]
+        if isinstance(sample_fname, str):
+            fs, _ = get_fs_and_path(sample_fname)
+
+            read_datasource_fn_kwargs = {
+                "paths": list(zip(fnames, idxs)),
+                "filesystem": PyFileSystem(FSSpecHandler(fs)),
+            }
+            if self.df_engine.partitioned and file_size is not None:
+                # Heuristic to determine parallelism: if the average file size is known (in bytes), then we can
+                # extrapolate to determine the total file size. We aim to have ~50MB partitions (5e7 bytes), so we
+                # set parallelism to be the total size / 50MB.
+                total_size = file_size * len(fnames)
+                parallelism = int(total_size / 5e7)
+                # Only set parallelism if it matches or exceeds the Ray default kwarg for parallelism
+                read_datasource_fn_kwargs["parallelism"] = max(RAY_DEFAULT_PARALLELISM, parallelism)
+
+            # The resulting column is named "value"
+            ds = ray.data.read_datasource(BinaryIgnoreNoneTypeDatasource(), **read_datasource_fn_kwargs)
+            ds = ds.add_column("idx", lambda df: df["value"].map(lambda row: int(row["idx"])))
+            # Overwrite the "value" column with the actual data
+            ds = ds.add_column("value", lambda df: df["value"].map(lambda row: row["data"]))
+        else:
+            # Assume the path has already been read in, so just convert directly to a dataset
+            # Name the column "value" to match the behavior of the above
+            column_df = column.to_frame(name="value")
+            column_df["idx"] = column_df.index
+            ds = self.df_engine.to_ray_dataset(column_df)
 
         def map_batches_fn(df: pd.DataFrame, fn: Callable) -> pd.DataFrame:
-            df[column.name] = df[column.name].map(fn)
-            return df
+            # HACK: Workaround for https://github.com/modin-project/modin/issues/4686
+            if "value" in df:
+                key = "value"
+            else:
+                key = column.name
+
+            # We need to explicitly pass the credentials stored in fsspec.conf since the operation occurs on Ray.
+            with use_credentials(conf):
+                df[key] = df[key].map(fn)
+                return df
 
-        ds = ds.map_batches(partial(map_batches_fn, fn=get_bytes_obj_if_path), batch_format="pandas")
         if map_fn is not None:
             ds = ds.map_batches(partial(map_batches_fn, fn=map_fn), batch_format="pandas")
 
-        return self.df_engine.from_ray_dataset(ds)[column.name]
+        df = self.df_engine.from_ray_dataset(ds).rename(columns={"value": column.name})
+        if "idx" in df.columns:
+            df = df.set_index("idx", drop=True)
+            df = self.df_engine.map_partitions(
+                df, lambda pd_df: set_index_name(pd_df, column.index.name), meta={column.name: "object"}
+            )
+        return df[column.name]
 
     @property
     def num_nodes(self) -> int:
         if not ray.is_initialized():
             return 1
         return len(ray.nodes())
+
+    def get_available_resources(self) -> Resources:
+        resources = ray.cluster_resources()
+        return Resources(cpus=resources.get("CPU", 0), gpus=resources.get("GPU", 0))
+
+
+def initialize_ray():
+    if not ray.is_initialized():
+        try:
+            ray.init("auto", ignore_reinit_error=True)
+        except ConnectionError:
+            init_ray_local()
+
+
+def init_ray_local():
+    logger.info("Initializing new Ray cluster...")
+    ray.init(ignore_reinit_error=True)
