@@ -14,15 +14,17 @@
 # limitations under the License.
 # ==============================================================================
 import os
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Optional, Union
 
 import numpy as np
 import pandas as pd
 import torch
 
 from ludwig.backend import LOCAL_BACKEND
+from ludwig.data.utils import convert_to_dict
 from ludwig.utils.data_utils import DATAFRAME_FORMATS, DICT_FORMATS
 from ludwig.utils.dataframe_utils import to_numpy_dataset
+from ludwig.utils.fs_utils import has_remote_protocol, open_file
 from ludwig.utils.misc_utils import get_from_registry
 from ludwig.utils.strings_utils import make_safe_filename
 
@@ -43,13 +45,15 @@ def postprocess(
     if not skip_save_unprocessed_output:
         _save_as_numpy(predictions, output_directory, saved_keys, backend)
 
-    for of_name, output_feature in output_features.items():
-        predictions = output_feature.postprocess_predictions(
-            predictions,
-            training_set_metadata[of_name],
-            output_directory=output_directory,
-            backend=backend,
-        )
+    def postprocess_batch(df):
+        for of_name, output_feature in output_features.items():
+            df = output_feature.postprocess_predictions(
+                df,
+                training_set_metadata[of_name],
+            )
+        return df
+
+    predictions = backend.df_engine.map_batches(predictions, postprocess_batch)
 
     # Save any new columns but do not save the original columns again
     if not skip_save_unprocessed_output:
@@ -65,7 +69,11 @@ def _save_as_numpy(predictions, output_directory, saved_keys, backend):
     for k, v in numpy_predictions.items():
         k = k.replace("<", "[").replace(">", "]")  # Replace <UNK> and <PAD> with [UNK], [PAD]
         if k not in saved_keys:
-            np.save(npy_filename.format(make_safe_filename(k)), v)
+            if has_remote_protocol(output_directory):
+                with open_file(npy_filename.format(make_safe_filename(k)), mode="wb") as f:
+                    np.save(f, v)
+            else:
+                np.save(npy_filename.format(make_safe_filename(k)), v)
             saved_keys.add(k)
 
 
@@ -103,39 +111,17 @@ def convert_dict_to_df(predictions: Dict[str, Dict[str, Union[List[Any], torch.T
     return pd.DataFrame.from_dict(output)
 
 
-def convert_predictions(predictions, output_features, return_type="dict"):
-    convert_fn = get_from_registry(return_type, conversion_registry)
-    return convert_fn(
-        predictions,
-        output_features,
-    )
-
-
-def convert_to_dict(
-    predictions,
-    output_features,
+def convert_predictions(
+    predictions, output_features, return_type="dict", backend: Optional["Backend"] = None  # noqa: F821
 ):
-    output = {}
-    for of_name, output_feature in output_features.items():
-        feature_keys = {k for k in predictions.columns if k.startswith(of_name)}
-        feature_dict = {}
-        for key in feature_keys:
-            subgroup = key[len(of_name) + 1 :]
-
-            values = predictions[key]
-            try:
-                values = np.stack(values.to_numpy())
-            except ValueError:
-                values = values.to_list()
-
-            feature_dict[subgroup] = values
-        output[of_name] = feature_dict
-    return output
+    convert_fn = get_from_registry(return_type, conversion_registry)
+    return convert_fn(predictions, output_features, backend)
 
 
 def convert_to_df(
     predictions,
     output_features,
+    backend: Optional["Backend"] = None,  # noqa: F821
 ):
     return predictions
 
