@@ -15,6 +15,7 @@
 # ==============================================================================
 import logging
 import os
+import warnings
 from collections import Counter
 from functools import partial
 from typing import Any, Dict, List, Optional, Tuple, Union
@@ -24,29 +25,31 @@ import torch
 import torchvision
 
 from ludwig.constants import (
-    BACKFILL,
     CHECKSUM,
     COLUMN,
+    ENCODER,
     HEIGHT,
     IMAGE,
     INFER_IMAGE_DIMENSIONS,
     INFER_IMAGE_MAX_HEIGHT,
     INFER_IMAGE_MAX_WIDTH,
     INFER_IMAGE_SAMPLE_SIZE,
-    MISSING_VALUE_STRATEGY_OPTIONS,
     NAME,
     NUM_CHANNELS,
     PREPROCESSING,
     PROC_COLUMN,
-    RESIZE_METHODS,
     SRC,
     TIED,
     TRAINING,
+    TYPE,
     WIDTH,
 )
 from ludwig.data.cache.types import wrap
 from ludwig.features.base_feature import BaseFeatureMixin, InputFeature
+from ludwig.schema.features.image_feature import ImageInputFeatureConfig
+from ludwig.schema.features.utils import register_input_feature
 from ludwig.utils.data_utils import get_abs_path
+from ludwig.utils.dataframe_utils import is_dask_series_or_df
 from ludwig.utils.fs_utils import has_remote_protocol, upload_h5
 from ludwig.utils.image_utils import (
     get_gray_default_image,
@@ -56,9 +59,12 @@ from ludwig.utils.image_utils import (
     read_image_from_path,
     resize_image,
 )
-from ludwig.utils.misc_utils import set_default_value
+from ludwig.utils.misc_utils import set_default_value, set_default_values
 from ludwig.utils.types import Series, TorchscriptPreprocessingInput
 
+logger = logging.getLogger(__name__)
+
+
 # TODO(shreya): Confirm if it's ok to do per channel normalization
 # TODO(shreya): Also confirm if this is being used anywhere
 # TODO(shreya): Confirm if ok to use imagenet means and std devs
@@ -125,36 +131,7 @@ class ImageFeatureMixin(BaseFeatureMixin):
 
     @staticmethod
     def preprocessing_defaults():
-        return {
-            "missing_value_strategy": BACKFILL,
-            "in_memory": True,
-            "resize_method": "interpolate",
-            "scaling": "pixel_normalization",
-            "num_processes": 1,
-            "infer_image_num_channels": True,
-            "infer_image_dimensions": True,
-            "infer_image_max_height": 256,
-            "infer_image_max_width": 256,
-            "infer_image_sample_size": 100,
-        }
-
-    @staticmethod
-    def preprocessing_schema():
-        return {
-            "missing_value_strategy": {"type": "string", "enum": MISSING_VALUE_STRATEGY_OPTIONS},
-            "in_memory": {"type": "boolean"},
-            "resize_method": {"type": "string", "enum": RESIZE_METHODS},
-            "scaling": {"type": "string", "enum": list(image_scaling_registry.keys())},
-            "num_processes": {"type": "integer", "minimum": 0},
-            "height": {"type": "integer", "minimum": 0},
-            "width": {"type": "integer", "minimum": 0},
-            "num_channels": {"type": "integer", "minimum": 0},
-            "infer_image_num_channels": {"type": "boolean"},
-            "infer_image_dimensions": {"type": "boolean"},
-            "infer_image_max_height": {"type": "integer", "minimum": 0},
-            "infer_image_max_width": {"type": "integer", "minimum": 0},
-            "infer_image_sample_size": {"type": "integer", "minimum": 0},
-        }
+        return ImageInputFeatureConfig().preprocessing.to_dict()
 
     @staticmethod
     def cast_column(column, backend):
@@ -198,12 +175,12 @@ class ImageFeatureMixin(BaseFeatureMixin):
         if isinstance(img_entry, bytes):
             img = read_image_from_bytes_obj(img_entry, num_channels)
         elif isinstance(img_entry, np.ndarray):
-            img = torch.from_numpy(img_entry).permute(2, 0, 1)
+            img = torch.from_numpy(np.array(img_entry, copy=True)).permute(2, 0, 1)
         else:
             img = img_entry
 
         if not isinstance(img, torch.Tensor):
-            logging.info(f"Image with value {img} cannot be read")
+            warnings.warn(f"Image with value {img} cannot be read")
             return None
 
         img_num_channels = num_channels_in_image(img)
@@ -227,7 +204,7 @@ class ImageFeatureMixin(BaseFeatureMixin):
                 img = torch.nn.functional.pad(img, [0, 0, 0, 0, 0, extra_channels])
 
             if img_num_channels != num_channels:
-                logging.warning(
+                logger.warning(
                     "Image has {} channels, where as {} "
                     "channels are expected. Dropping/adding channels "
                     "with 0s as appropriate".format(img_num_channels, num_channels)
@@ -275,7 +252,7 @@ class ImageFeatureMixin(BaseFeatureMixin):
         height = min(int(round(height_avg)), max_height)
         width = min(int(round(width_avg)), max_width)
 
-        logging.debug(f"Inferring height: {height} and width: {width}")
+        logger.debug(f"Inferring height: {height} and width: {width}")
         return height, width
 
     @staticmethod
@@ -300,16 +277,16 @@ class ImageFeatureMixin(BaseFeatureMixin):
         else:
             # Default case: use 3 channels.
             num_channels = 3
-        logging.info(f"Inferring num_channels from the first {n_images} images.")
-        logging.info("\n".join([f"  images with {k} channels: {v}" for k, v in sorted(channel_frequency.items())]))
+        logger.info(f"Inferring num_channels from the first {n_images} images.")
+        logger.info("\n".join([f"  images with {k} channels: {v}" for k, v in sorted(channel_frequency.items())]))
         if num_channels == max(channel_frequency, key=channel_frequency.get):
-            logging.info(
+            logger.info(
                 f"Using {num_channels} channels because it is the majority in sample. If an image with"
                 f" a different depth is read, will attempt to convert to {num_channels} channels."
             )
         else:
-            logging.info(f"Defaulting to {num_channels} channels.")
-        logging.info(
+            logger.info(f"Defaulting to {num_channels} channels.")
+        logger.info(
             "To explicitly set the number of channels, define num_channels in the preprocessing dictionary of "
             "the image input feature config."
         )
@@ -327,20 +304,23 @@ class ImageFeatureMixin(BaseFeatureMixin):
         expected be of the same size with the same number of channels
         """
 
-        explicit_height_width = HEIGHT in preprocessing_parameters or WIDTH in preprocessing_parameters
+        explicit_height_width = preprocessing_parameters[HEIGHT] or preprocessing_parameters[WIDTH]
         explicit_num_channels = NUM_CHANNELS in preprocessing_parameters and preprocessing_parameters[NUM_CHANNELS]
 
-        sample = []
         if preprocessing_parameters[INFER_IMAGE_DIMENSIONS] and not (explicit_height_width and explicit_num_channels):
             sample_size = min(len(column), preprocessing_parameters[INFER_IMAGE_SAMPLE_SIZE])
         else:
             sample_size = 1  # Take first image
 
+        sample = []
+        sample_num_bytes = []
         failed_entries = []
         for image_entry in column.head(sample_size):
             if isinstance(image_entry, str):
                 # Tries to read image as PNG or numpy file from the path.
-                image = read_image_from_path(image_entry)
+                image, num_bytes = read_image_from_path(image_entry, return_num_bytes=True)
+                if num_bytes is not None:
+                    sample_num_bytes.append(num_bytes)
             else:
                 image = image_entry
 
@@ -401,7 +381,9 @@ class ImageFeatureMixin(BaseFeatureMixin):
                 )
 
         assert isinstance(num_channels, int), ValueError("Number of image channels needs to be an integer")
-        return (should_resize, width, height, num_channels, user_specified_num_channels)
+
+        average_file_size = np.mean(sample_num_bytes) if sample_num_bytes else None
+        return (should_resize, width, height, num_channels, user_specified_num_channels, average_file_size)
 
     @staticmethod
     def add_feature_data(
@@ -426,6 +408,7 @@ class ImageFeatureMixin(BaseFeatureMixin):
             height,
             num_channels,
             user_specified_num_channels,
+            average_file_size,
         ) = ImageFeatureMixin._finalize_preprocessing_parameters(preprocessing_parameters, abs_path_column)
 
         metadata[name][PREPROCESSING]["height"] = height
@@ -453,11 +436,22 @@ class ImageFeatureMixin(BaseFeatureMixin):
         if in_memory or skip_save_processed_input:
             metadata[name]["reshape"] = (num_channels, height, width)
 
-            proc_col = backend.read_binary_files(abs_path_column, map_fn=read_image_if_bytes_obj_and_resize)
-            proc_col = backend.df_engine.map_objects(proc_col, lambda row: row if row is not None else default_image)
+            proc_col = backend.read_binary_files(
+                abs_path_column, map_fn=read_image_if_bytes_obj_and_resize, file_size=average_file_size
+            )
+
+            num_failed_image_reads = (
+                proc_col.isna().sum().compute() if is_dask_series_or_df(proc_col, backend) else proc_col.isna().sum()
+            )
+
+            proc_col = backend.df_engine.map_objects(
+                proc_col, lambda row: default_image if not isinstance(row, np.ndarray) else row
+            )
+
             proc_df[feature_config[PROC_COLUMN]] = proc_col
         else:
             num_images = len(abs_path_column)
+            num_failed_image_reads = 0
 
             data_fp = backend.cache.get_cache_path(wrap(metadata.get(SRC)), metadata.get(CHECKSUM), TRAINING)
             with upload_h5(data_fp) as h5_file:
@@ -467,27 +461,34 @@ class ImageFeatureMixin(BaseFeatureMixin):
                 )
                 for i, img_entry in enumerate(abs_path_column):
                     res = read_image_if_bytes_obj_and_resize(img_entry)
-                    image_dataset[i, :height, :width, :] = res if res is not None else default_image
+                    if isinstance(res, np.ndarray):
+                        image_dataset[i, :height, :width, :] = res
+                    else:
+                        image_dataset[i, :height, :width, :] = default_image
+                        num_failed_image_reads += 1
                 h5_file.flush()
 
             proc_df[feature_config[PROC_COLUMN]] = np.arange(num_images)
+
+        if num_failed_image_reads > 0:
+            logger.warning(
+                f"Failed to read {num_failed_image_reads} images while preprocessing feature `{name}`. "
+                "Using default image for these rows in the dataset."
+            )
+
         return proc_df
 
 
+@register_input_feature(IMAGE)
 class ImageInputFeature(ImageFeatureMixin, InputFeature):
-    height = 0
-    width = 0
-    num_channels = 0
-    scaling = "pixel_normalization"
-    encoder = "stacked_cnn"
+    def __init__(self, input_feature_config: Union[ImageInputFeatureConfig, Dict], encoder_obj=None, **kwargs):
+        input_feature_config = self.load_config(input_feature_config)
+        super().__init__(input_feature_config, **kwargs)
 
-    def __init__(self, feature, encoder_obj=None):
-        super().__init__(feature)
-        self.overwrite_defaults(feature)
         if encoder_obj:
             self.encoder_obj = encoder_obj
         else:
-            self.encoder_obj = self.initialize_encoder(feature)
+            self.encoder_obj = self.initialize_encoder(input_feature_config.encoder)
 
     def forward(self, inputs: torch.Tensor) -> torch.Tensor:
         assert isinstance(inputs, torch.Tensor)
@@ -506,7 +507,9 @@ class ImageInputFeature(ImageFeatureMixin, InputFeature):
 
     @property
     def input_shape(self) -> torch.Size:
-        return torch.Size([self.num_channels, self.height, self.width])
+        return torch.Size(
+            [self.encoder_obj.config.num_channels, self.encoder_obj.config.height, self.encoder_obj.config.width]
+        )
 
     @property
     def output_shape(self) -> torch.Size:
@@ -515,12 +518,18 @@ class ImageInputFeature(ImageFeatureMixin, InputFeature):
     @staticmethod
     def update_config_with_metadata(input_feature, feature_metadata, *args, **kwargs):
         for key in ["height", "width", "num_channels", "scaling"]:
-            input_feature[key] = feature_metadata[PREPROCESSING][key]
+            input_feature[ENCODER][key] = feature_metadata[PREPROCESSING][key]
 
     @staticmethod
     def populate_defaults(input_feature):
-        set_default_value(input_feature, TIED, None)
+        defaults = ImageInputFeatureConfig()
+        set_default_value(input_feature, TIED, defaults.tied)
         set_default_value(input_feature, PREPROCESSING, {})
+        set_default_values(input_feature, {ENCODER: {TYPE: defaults.encoder.type}})
+
+    @staticmethod
+    def get_schema_cls():
+        return ImageInputFeatureConfig
 
     @staticmethod
     def create_preproc_module(metadata: Dict[str, Any]) -> torch.nn.Module:
