@@ -1,19 +1,43 @@
+import copy
 import logging
 import os
+import warnings
 from pprint import pformat
-from typing import List, Optional, Union
+from typing import Any, Dict, List, Optional, Union
 
 import pandas as pd
 import yaml
+from tabulate import tabulate
 
 from ludwig.api import LudwigModel
 from ludwig.backend import Backend, initialize_backend, LocalBackend
 from ludwig.callbacks import Callback
-from ludwig.constants import COMBINED, EXECUTOR, HYPEROPT, LOSS, MINIMIZE, TEST, TRAINING, TYPE, VALIDATION
+from ludwig.constants import (
+    COMBINED,
+    EXECUTOR,
+    GOAL,
+    GRID_SEARCH,
+    HYPEROPT,
+    LOSS,
+    METRIC,
+    MINIMIZE,
+    NAME,
+    OUTPUT_FEATURES,
+    PREPROCESSING,
+    RAY,
+    SPACE,
+    SPLIT,
+    TEST,
+    TRAINING,
+    TYPE,
+    VALIDATION,
+)
+from ludwig.data.split import get_splitter
 from ludwig.features.feature_registries import output_type_registry
-from ludwig.hyperopt.execution import executor_registry, get_build_hyperopt_executor, RayTuneExecutor
 from ludwig.hyperopt.results import HyperoptResults
 from ludwig.hyperopt.utils import print_hyperopt_results, save_hyperopt_stats, should_tune_preprocessing
+from ludwig.utils.backward_compatibility import upgrade_to_latest_version
+from ludwig.utils.dataset_utils import generate_dataset_statistics
 from ludwig.utils.defaults import default_random_seed, merge_with_defaults
 from ludwig.utils.fs_utils import makedirs, open_file
 from ludwig.utils.misc_utils import get_class_attributes, get_from_registry, set_default_value, set_default_values
@@ -26,6 +50,9 @@ except ImportError:
         pass
 
 
+logger = logging.getLogger(__name__)
+
+
 def hyperopt(
     config: Union[str, dict],
     dataset: Union[str, dict, pd.DataFrame] = None,
@@ -161,6 +188,8 @@ def hyperopt(
     :return: (List[dict]) List of results for each trial, ordered by
         descending performance on the target metric.
     """
+    from ludwig.hyperopt.execution import get_build_hyperopt_executor, RayTuneExecutor
+
     # check if config is a path or a dict
     if isinstance(config, str):  # assume path
         with open_file(config, "r") as def_file:
@@ -168,33 +197,44 @@ def hyperopt(
     else:
         config_dict = config
 
+    # backwards compatibility
+    config = upgrade_to_latest_version(config_dict)
+
+    # Retain pre-merged config for hyperopt schema generation
+    premerged_config = copy.deepcopy(config)
+
     # merge config with defaults
-    config = merge_with_defaults(config_dict)
+    config = merge_with_defaults(config)
 
     if HYPEROPT not in config:
         raise ValueError("Hyperopt Section not present in config")
 
-    hyperopt_config = config["hyperopt"]
+    hyperopt_config = config[HYPEROPT]
 
     update_hyperopt_params_with_defaults(hyperopt_config)
 
-    # print hyperopt config
-    logging.info(pformat(hyperopt_config, indent=4))
-    logging.info("\n")
+    # Print hyperopt config
+    logger.info("Hyperopt config")
+    logger.info(pformat(hyperopt_config, indent=4))
+    logger.info("\n")
 
     search_alg = hyperopt_config["search_alg"]
-    executor = hyperopt_config["executor"]
+    executor = hyperopt_config[EXECUTOR]
     parameters = hyperopt_config["parameters"]
     split = hyperopt_config["split"]
     output_feature = hyperopt_config["output_feature"]
     metric = hyperopt_config["metric"]
     goal = hyperopt_config["goal"]
 
+    # Check if all features are grid type parameters and log UserWarning if needed
+    log_warning_if_all_grid_type_parameters(parameters, executor.get("num_samples"))
+
     ######################
     # check validity of output_feature / metric/ split combination
     ######################
+    splitter = get_splitter(**config[PREPROCESSING]["split"])
     if split == TRAINING:
-        if training_set is None and (config["preprocessing"]["split_probabilities"][0] <= 0):
+        if training_set is None and not splitter.has_split(0):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
                 "was not provided, "
@@ -202,7 +242,7 @@ def hyperopt(
                 "of the config is not greater than 0".format(split)
             )
     elif split == VALIDATION:
-        if validation_set is None and (config["preprocessing"]["split_probabilities"][1] <= 0):
+        if validation_set is None and not splitter.has_split(1):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
                 "was not provided, "
@@ -210,7 +250,7 @@ def hyperopt(
                 "of the config is not greater than 0".format(split)
             )
     elif split == TEST:
-        if test_set is None and (config["preprocessing"]["split_probabilities"][2] <= 0):
+        if test_set is None and not splitter.has_split(2):
             raise ValueError(
                 'The data for the specified split for hyperopt "{}" '
                 "was not provided, "
@@ -225,7 +265,7 @@ def hyperopt(
         if metric != LOSS:
             raise ValueError('The only valid metric for "combined" output feature is "loss"')
     else:
-        output_feature_names = {of["name"] for of in config["output_features"]}
+        output_feature_names = {of[NAME] for of in config[OUTPUT_FEATURES]}
         if output_feature not in output_feature_names:
             raise ValueError(
                 'The output feature specified for hyperopt "{}" '
@@ -234,14 +274,14 @@ def hyperopt(
             )
 
         output_feature_type = None
-        for of in config["output_features"]:
-            if of["name"] == output_feature:
+        for of in config[OUTPUT_FEATURES]:
+            if of[NAME] == output_feature:
                 output_feature_type = of[TYPE]
         feature_class = get_from_registry(output_feature_type, output_type_registry)
         if metric not in feature_class.metric_functions:
             # todo v0.4: allow users to specify also metrics from the overall
-            #  and per class metrics from the trainign stats and in general
-            #  and potprocessed metric
+            #  and per class metrics from the training stats and in general
+            #  and post-processed metric
             raise ValueError(
                 'The specified metric for hyperopt "{}" is not a valid metric '
                 'for the specified output feature "{}" of type "{}". '
@@ -295,6 +335,11 @@ def hyperopt(
         )
         dataset = None
 
+        dataset_statistics = generate_dataset_statistics(training_set, validation_set, test_set)
+
+        logger.info("\nDataset Statistics")
+        logger.info(tabulate(dataset_statistics, headers="firstrow", tablefmt="fancy_grid"))
+
         for callback in callbacks or []:
             callback.on_hyperopt_preprocessing_end(experiment_name)
 
@@ -302,7 +347,7 @@ def hyperopt(
         callback.on_hyperopt_start(experiment_name)
 
     hyperopt_results = hyperopt_executor.execute(
-        config,
+        premerged_config,
         dataset=dataset,
         training_set=training_set,
         validation_set=validation_set,
@@ -345,25 +390,49 @@ def hyperopt(
             }
 
             save_hyperopt_stats(hyperopt_stats, results_directory)
-            logging.info(f"Hyperopt stats saved to: {results_directory}")
+            logger.info(f"Hyperopt stats saved to: {results_directory}")
 
     for callback in callbacks or []:
         callback.on_hyperopt_end(experiment_name)
         callback.on_hyperopt_finish(experiment_name)
 
-    logging.info("Finished hyperopt")
+    logger.info("Finished hyperopt")
 
     return hyperopt_results
 
 
-def update_hyperopt_params_with_defaults(hyperopt_params):
-    set_default_value(hyperopt_params, EXECUTOR, {})
-    set_default_value(hyperopt_params, "split", VALIDATION)
-    set_default_value(hyperopt_params, "output_feature", COMBINED)
-    set_default_value(hyperopt_params, "metric", LOSS)
-    set_default_value(hyperopt_params, "goal", MINIMIZE)
+def log_warning_if_all_grid_type_parameters(hyperopt_parameter_config: Dict[str, Any], num_samples: int = 1) -> None:
+    """Logs warning if all parameters have a grid type search space and num_samples > 1 since this will result in
+    duplicate trials being created."""
+    if num_samples == 1:
+        return
 
-    set_default_values(hyperopt_params[EXECUTOR], {TYPE: "ray"})
+    total_grid_search_trials = 1
+
+    for _, param_info in hyperopt_parameter_config.items():
+        if param_info.get(SPACE, None) != GRID_SEARCH:
+            return
+        total_grid_search_trials *= len(param_info.get("values", []))
+
+    num_duplicate_trials = (total_grid_search_trials * num_samples) - total_grid_search_trials
+    warnings.warn(
+        "All hyperopt parameters in Ludwig config are using grid_search space, but number of samples "
+        f"({num_samples}) is greater than 1. This will result in {num_duplicate_trials} duplicate trials being "
+        "created. Consider setting `num_samples` to 1 in the hyperopt executor to prevent trial duplication.",
+        RuntimeWarning,
+    )
+
+
+def update_hyperopt_params_with_defaults(hyperopt_params):
+    from ludwig.hyperopt.execution import executor_registry
+
+    set_default_value(hyperopt_params, EXECUTOR, {})
+    set_default_value(hyperopt_params, SPLIT, VALIDATION)
+    set_default_value(hyperopt_params, "output_feature", COMBINED)
+    set_default_value(hyperopt_params, METRIC, LOSS)
+    set_default_value(hyperopt_params, GOAL, MINIMIZE)
+
+    set_default_values(hyperopt_params[EXECUTOR], {TYPE: RAY, "num_samples": 1})
     executor = get_from_registry(hyperopt_params[EXECUTOR][TYPE], executor_registry)
     executor_defaults = {k: v for k, v in executor.__dict__.items() if k in get_class_attributes(executor)}
     set_default_values(
