@@ -23,15 +23,18 @@ import pathlib
 import shutil
 import tempfile
 import uuid
-from typing import Any, Optional, Union
+from typing import Optional
 from urllib.parse import unquote, urlparse
 
+import certifi
 import fsspec
 import h5py
-import requests
+import urllib3
 from filelock import FileLock
 from fsspec.core import split_protocol
 
+logger = logging.getLogger(__name__)
+
 
 def get_fs_and_path(url):
     protocol, path = split_protocol(url)
@@ -60,44 +63,48 @@ def upgrade_http(urlpath):
     return None
 
 
-def get_bytes_obj_if_path(path: Any) -> Union[Any, Optional[bytes]]:
-    """Gets bytes string if `path` is a path (e.g. a string).
-
-    If it is not a path, return as-is.
-    """
-    if not isinstance(path, str):
-        return path
-    return get_bytes_obj_from_path(path)
-
-
 @functools.lru_cache(maxsize=32)
 def get_bytes_obj_from_path(path: str) -> Optional[bytes]:
     if is_http(path):
         try:
             return get_bytes_obj_from_http_path(path)
-        except requests.exceptions.RequestException as e:
-            logging.warning(e)
+        except Exception as e:
+            logger.warning(e)
             return None
     else:
         try:
             with open_file(path) as f:
                 return f.read()
         except OSError as e:
-            logging.warning(e)
+            logger.warning(e)
             return None
 
 
+def stream_http_get_request(path: str) -> urllib3.response.HTTPResponse:
+    if upgrade_http(path):
+        http = urllib3.PoolManager()
+    else:
+        http = urllib3.PoolManager(ca_certs=certifi.where())
+    resp = http.request("GET", path, preload_content=False)
+    return resp
+
+
 @functools.lru_cache(maxsize=32)
 def get_bytes_obj_from_http_path(path: str) -> bytes:
-    data = requests.get(path, stream=True)
-    if data.status_code == 404:
+    resp = stream_http_get_request(path)
+    if resp.status == 404:
         upgraded = upgrade_http(path)
         if upgraded:
-            logging.info(f"reading url {path} failed. upgrading to https and retrying")
+            logger.info(f"reading url {path} failed. upgrading to https and retrying")
             return get_bytes_obj_from_http_path(upgraded)
         else:
-            raise requests.exceptions.HTTPError(f"reading url {path} failed and cannot be upgraded to https")
-    return data.raw.read()
+            raise urllib3.exceptions.HTTPError(f"reading url {path} failed and cannot be upgraded to https")
+
+    # stream data
+    data = b""
+    for chunk in resp.stream(1024):
+        data += chunk
+    return data
 
 
 def find_non_existing_dir_by_adding_suffix(directory_name):
